@@ -40,6 +40,13 @@ namedb_new (const char *filename)
 	db->diff_skip = 0;
 	db->fd = NULL;
 
+	if (gettimeofday(&(db->diff_timestamp), NULL) != 0) {
+		log_msg(LOG_ERR, "unable to load %s: cannot initialize "
+						 "timestamp", db->filename);
+		region_destroy(region);
+		return NULL;
+	}
+
 	/*
 	 * Unlink the old database, if it exists.  This is useful to
 	 * ensure that NSD doesn't see the changes until a reload is done.
@@ -50,7 +57,7 @@ namedb_new (const char *filename)
 	}
 
 	/* Create the database */
-    if ((db->fd = fopen(db->filename, "w")) == NULL) {
+	if ((db->fd = fopen(db->filename, "w")) == NULL) {
 		region_destroy(region);
 		return NULL;
 	}
