@@ -114,6 +114,8 @@ typedef enum nsd_rc nsd_rc_type;
 #define TYPE_KX		36	/* RFC2230 Key Exchange Delegation Record */
 #define TYPE_CERT	37	/* RFC2538 */
 
+#define TYPE_A6		38	/* RFC2874 */
+
 #define TYPE_DNAME	39	/* RFC2672 */
 
 #define TYPE_OPT	41	/* Pseudo OPT record... */
@@ -158,6 +160,7 @@ enum rdata_wireformat
 {
 	RDATA_WF_COMPRESSED_DNAME,   /* Possibly compressed domain name.  */
 	RDATA_WF_UNCOMPRESSED_DNAME, /* Uncompressed domain name.  */
+	RDATA_WF_LITERAL_DNAME,      /* Literal (not downcased) dname.  */
 	RDATA_WF_BYTE,		     /* 8-bit integer.  */
 	RDATA_WF_SHORT,		     /* 16-bit integer.  */
 	RDATA_WF_LONG,		     /* 32-bit integer.  */
@@ -177,6 +180,7 @@ typedef enum rdata_wireformat rdata_wireformat_type;
 enum rdata_zoneformat
 {
 	RDATA_ZF_DNAME,		/* Domain name.  */
+	RDATA_ZF_LITERAL_DNAME,	/* DNS name (not lowercased domain name).  */
 	RDATA_ZF_TEXT,		/* Text string.  */
 	RDATA_ZF_BYTE,		/* 8-bit integer.  */
 	RDATA_ZF_SHORT,		/* 16-bit integer.  */
