@@ -41,8 +41,6 @@
  */
 #define MAX_WAITING_TIME TCP_TIMEOUT
 
-#define SERIAL_BITS      32
-
 /*
  * Exit codes are based on named-xfer for now.  See ns_defs.h in
  * bind8.
@@ -82,9 +80,7 @@ struct axfr_state
 	 * Region used to store owner and origin of previous RR (used
 	 * for pretty printing of zone data).
 	 */
-	region_type *previous_owner_region;
-	const dname_type *previous_owner;
-	const dname_type *previous_owner_origin;
+	struct state_pretty_rr *pretty_rr;
 };
 typedef struct axfr_state axfr_state_type;
 
@@ -113,6 +109,12 @@ int check_matching_address_family(struct addrinfo *a, struct addrinfo *b);
  */
 struct addrinfo *find_by_address_family(struct addrinfo *addrs, int family);
 
+/*
+ * Assigns pointers to hostname and port and wipes out the optional delimiter.
+ */
+void get_hostname_port_frm_str(char* arg, const char** hostname,
+	const char** port);
+
 /*
  * Log an error message and exit.
  */
@@ -151,19 +153,20 @@ usage (void)
 	fprintf(stderr,
 		"Usage: nsd-xfer [OPTION]... -z zone -f file server...\n"
 		"NSD AXFR client.\n\nSupported options:\n"
-		"  -4           Only use IPv4 connections.\n"
-		"  -6           Only use IPv6 connections.\n"
-		"  -a src       Local hostname/IP for the connection.\n"
-		"  -f file      Output zone file name.\n"
-		"  -p port      The port to connect to.\n"
-		"  -s serial    The current zone serial.\n"
-		"  -T tsiginfo  The TSIG key file name.  The file is removed "
+		"  -4            Only use IPv4 connections.\n"
+		"  -6            Only use IPv6 connections.\n"
+		"  -a src[@port] Local hostname/ip-address for the \
+connection, including optional source port.\n"
+		"  -f file       Output zone file name.\n"
+		"  -p port       The port to connect to.\n"
+		"  -s serial     The current zone serial.\n"
+		"  -T tsiginfo   The TSIG key file name.  The file is removed "
 		"after reading the\n               key.\n"
 		"  -v           Verbose output.\n");
 	fprintf(stderr,
 		"  -z zone      Specify the name of the zone to transfer.\n"
 		"  server       The name or IP address of the master server.\n"
-		"\nReport bugs to <%s>.\n", PACKAGE_BUGREPORT);
+		"\nVersion %s. Report bugs to <%s>.\n", PACKAGE_VERSION, PACKAGE_BUGREPORT);
 	exit(XFER_FAIL);
 }
 
@@ -183,12 +186,6 @@ to_alarm(int ATTR_UNUSED(sig))
 	timeout_flag = 1;
 }
 
-static void
-cleanup_addrinfo(void *data)
-{
-	freeaddrinfo((struct addrinfo *) data);
-}
-
 /*
  * Read a line from IN.  If successful, the line is stripped of
  * leading and trailing whitespace and non-zero is returned.
@@ -204,38 +201,27 @@ read_line(FILE *in, char *line, size_t size)
 	}
 }
 
+#ifdef TSIG
 static tsig_key_type *
-read_tsig_key_data(region_type *region, FILE *in, int default_family)
+read_tsig_key_data(region_type *region, FILE *in,
+	int ATTR_UNUSED(default_family), tsig_algorithm_type** tsig_algo)
 {
 	char line[4000];
 	tsig_key_type *key = (tsig_key_type *) region_alloc(
 		region, sizeof(tsig_key_type));
-	struct addrinfo hints;
-	int gai_rc;
 	int size;
+	uint8_t algo = 0;
 	uint8_t data[4000];
 
+	/* server address */
 	if (!read_line(in, line, sizeof(line))) {
 		error("failed to read TSIG key server address: '%s'",
 		      strerror(errno));
 		return NULL;
 	}
+	/* server address unused */
 
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_flags |= AI_NUMERICHOST;
-	hints.ai_family = default_family;
-	hints.ai_socktype = SOCK_STREAM;
-	hints.ai_protocol = IPPROTO_TCP;
-	gai_rc = getaddrinfo(line, NULL, &hints, &key->server);
-	if (gai_rc) {
-		error("cannot parse address '%s': %s",
-		      line,
-		      gai_strerror(gai_rc));
-		return NULL;
-	}
-
-	region_add_cleanup(region, cleanup_addrinfo, key->server);
-
+	/* tsig keyname */
 	if (!read_line(in, line, sizeof(line))) {
 		error("failed to read TSIG key name: '%s'", strerror(errno));
 		return NULL;
@@ -247,11 +233,19 @@ read_tsig_key_data(region_type *region, FILE *in, int default_family)
 		return NULL;
 	}
 
+	/* tsig algorithm */
 	if (!read_line(in, line, sizeof(line))) {
-		error("failed to read TSIG key type: '%s'", strerror(errno));
+		error("failed to read TSIG key algorithm: '%s'", strerror(errno));
+		return NULL;
+	}
+	algo = (uint8_t) atoi((const char*) line);
+	*tsig_algo = tsig_get_algorithm_by_id(algo);
+	if (*tsig_algo == NULL) {
+		error("failed to parse TSIG key algorithm %i: '%s'\n", algo, strerror(errno));
 		return NULL;
 	}
 
+	/* tsig secret */
 	if (!read_line(in, line, sizeof(line))) {
 		error("failed to read TSIG key data: '%s'\n", strerror(errno));
 		return NULL;
@@ -275,7 +269,7 @@ read_tsig_key_data(region_type *region, FILE *in, int default_family)
 static tsig_key_type *
 read_tsig_key(region_type *region,
 	      const char *tsiginfo_filename,
-	      int default_family)
+	      int default_family, tsig_algorithm_type** algo)
 {
 	FILE *in;
 	tsig_key_type *key;
@@ -288,7 +282,7 @@ read_tsig_key(region_type *region,
 		return NULL;
 	}
 
-	key = read_tsig_key_data(region, in, default_family);
+	key = read_tsig_key_data(region, in, default_family, algo);
 
 	fclose(in);
 
@@ -300,34 +294,7 @@ read_tsig_key(region_type *region,
 
 	return key;
 }
-
-/*
- * Write the complete buffer to the socket, irrespective of short
- * writes or interrupts.
- */
-static int
-write_socket(int s, const void *buf, size_t size)
-{
-	const char *data = (const char *) buf;
-	size_t total_count = 0;
-
-	while (total_count < size) {
-		ssize_t count
-			= write(s, data + total_count, size - total_count);
-		if (count == -1) {
-			if (errno != EAGAIN) {
-				error("network write failed: %s",
-				      strerror(errno));
-				return 0;
-			} else {
-				continue;
-			}
-		}
-		total_count += count;
-	}
-
-	return 1;
-}
+#endif /* TSIG */
 
 /*
  * Read SIZE bytes from the socket into BUF.  Keep reading unless an
@@ -361,110 +328,6 @@ read_socket(int s, void *buf, size_t size)
 	return 1;
 }
 
-static int
-print_rdata(buffer_type *output, rrtype_descriptor_type *descriptor,
-	    rr_type *record)
-{
-	size_t i;
-	size_t saved_position = buffer_position(output);
-
-	for (i = 0; i < record->rdata_count; ++i) {
-		if (i == 0) {
-			buffer_printf(output, "\t");
-		} else if (descriptor->type == TYPE_SOA && i == 2) {
-			buffer_printf(output, " (\n\t\t");
-		} else {
-			buffer_printf(output, " ");
-		}
-		if (!rdata_atom_to_string(
-			    output,
-			    (rdata_zoneformat_type) descriptor->zoneformat[i],
-			    record->rdatas[i]))
-		{
-			buffer_set_position(output, saved_position);
-			return 0;
-		}
-	}
-	if (descriptor->type == TYPE_SOA) {
-		buffer_printf(output, " )");
-	}
-
-	return 1;
-}
-
-static void
-set_previous_owner(axfr_state_type *state, const dname_type *dname)
-{
-	region_free_all(state->previous_owner_region);
-	state->previous_owner = dname_copy(state->previous_owner_region, dname);
-	state->previous_owner_origin = dname_origin(
-		state->previous_owner_region, state->previous_owner);
-}
-
-static int
-print_rr(FILE *out,
-	 axfr_state_type *state,
-	 rr_type *record)
-{
-	buffer_type *output = buffer_create(state->rr_region, 1000);
-	rrtype_descriptor_type *descriptor
-		= rrtype_descriptor_by_type(record->type);
-	int result;
-	const dname_type *owner = domain_dname(record->owner);
-	const dname_type *owner_origin
-		= dname_origin(state->rr_region, owner);
-	int owner_changed
-		= (!state->previous_owner
-		   || dname_compare(state->previous_owner, owner) != 0);
-	if (owner_changed) {
-		int origin_changed = (!state->previous_owner_origin
-				      || dname_compare(
-					      state->previous_owner_origin,
-					      owner_origin) != 0);
-		if (origin_changed) {
-			buffer_printf(
-				output,
-				"$ORIGIN %s\n",
-				dname_to_string(owner_origin, NULL));
-		}
-
-		set_previous_owner(state, owner);
-		buffer_printf(output,
-			      "%s",
-			      dname_to_string(owner,
-					      state->previous_owner_origin));
-	}
-
-	buffer_printf(output,
-		      "\t%lu\t%s\t%s",
-		      (unsigned long) record->ttl,
-		      rrclass_to_string(record->klass),
-		      rrtype_to_string(record->type));
-
-	result = print_rdata(output, descriptor, record);
-	if (!result) {
-		/*
-		 * Some RDATA failed to print, so print the record's
-		 * RDATA in unknown format.
-		 */
-		result = rdata_atoms_to_unknown_string(output,
-						       descriptor,
-						       record->rdata_count,
-						       record->rdatas);
-	}
-
-	if (result) {
-		buffer_printf(output, "\n");
-		buffer_flip(output);
-		(void)fwrite(buffer_current(output), buffer_remaining(output), 1,
-		       out);
-/* 		fflush(out); */
-	}
-
-	return result;
-}
-
-
 static int
 parse_response(FILE *out, axfr_state_type *state)
 {
@@ -507,7 +370,7 @@ parse_response(FILE *out, axfr_state_type *state)
 
 		++state->rr_count;
 
-		if (!print_rr(out, state, record)) {
+		if (!print_rr(out, state->pretty_rr, record)) {
 			return 0;
 		}
 
@@ -523,10 +386,12 @@ send_query(int s, query_type *q)
 	uint16_t size = htons(buffer_remaining(q->packet));
 
 	if (!write_socket(s, &size, sizeof(size))) {
+		error("network write failed: %s", strerror(errno));
 		return 0;
 	}
 	if (!write_socket(s, buffer_begin(q->packet), buffer_limit(q->packet)))
 	{
+		error("network write failed: %s", strerror(errno));
 		return 0;
 	}
 	return 1;
@@ -620,26 +485,6 @@ check_response_tsig(query_type *q, tsig_record_type *tsig)
 }
 
 
-/*
- * Compares two 32-bit serial numbers as defined in RFC1982.  Returns
- * <0 if a < b, 0 if a == b, and >0 if a > b.  The result is undefined
- * if a != b but neither is greater or smaller (see RFC1982 section
- * 3.2.).
- */
-static int
-compare_serial(uint32_t a, uint32_t b)
-{
-	const uint32_t cutoff = ((uint32_t) 1 << (SERIAL_BITS - 1));
-
-	if (a == b) {
-		return 0;
-	} else if ((a < b && b - a < cutoff) || (a > b && a - b > cutoff)) {
-		return -1;
-	} else {
-		return 1;
-	}
-}
-
 /*
  * Query the server for the zone serial. Return 1 if the zone serial
  * is higher than the current serial, 0 if the zone serial is lower or
@@ -925,13 +770,6 @@ print_stats(axfr_state_type *state)
 		(unsigned long) state->packets_received);
 }
 
-static void
-cleanup_region(void *data)
-{
-	region_type *region = (region_type *) data;
-	region_destroy(region);
-}
-
 int
 main(int argc, char *argv[])
 {
@@ -943,6 +781,7 @@ main(int argc, char *argv[])
 	const char *local_hostname = NULL;
 	struct addrinfo *local_address, *local_addresses = NULL;
 	const char *port = TCP_PORT;
+	const char *local_port = NULL;
 	int default_family = DEFAULT_AI_FAMILY;
 	struct sigaction mysigaction;
 	FILE *zone_file;
@@ -970,11 +809,8 @@ main(int argc, char *argv[])
 	state.done = 0;
 	state.rr_count = 0;
 	state.rr_region = region_create(xalloc, free);
-	state.previous_owner_region = region_create(xalloc, free);
-	state.previous_owner = NULL;
-	state.previous_owner_origin = NULL;
+	state.pretty_rr = create_pretty_rr(region);
 
-	region_add_cleanup(region, cleanup_region, state.previous_owner_region);
 	region_add_cleanup(region, cleanup_region, state.rr_region);
 
 	srandom((unsigned long) getpid() * (unsigned long) time(NULL));
@@ -997,7 +833,8 @@ main(int argc, char *argv[])
 #endif /* !INET6 */
 			break;
 		case 'a':
-			local_hostname = optarg;
+			get_hostname_port_frm_str((char *) optarg,
+				&local_hostname, &local_port);
 			break;
 		case 'f':
 			zone_filename = optarg;
@@ -1019,8 +856,12 @@ main(int argc, char *argv[])
 			break;
 		}
 		case 'T':
+#ifdef TSIG
 			tsig_key_filename = optarg;
 			break;
+#else
+			log_msg(LOG_ERR, "option -T given but TSIG not enabled");
+#endif /* TSIG */
 		case 'v':
 			++state.verbose;
 			break;
@@ -1041,18 +882,13 @@ main(int argc, char *argv[])
 	if (argc == 0 || !zone_filename || !state.zone)
 		usage();
 
+#ifdef TSIG
 	if (tsig_key_filename) {
-		tsig_algorithm_type *md5
-			= tsig_get_algorithm_by_name("hmac-md5");
-		if (!md5) {
-			error("cannot initialize hmac-md5: TSIG support not"
-			      " enabled");
-			exit(XFER_FAIL);
-		}
-
+		tsig_algorithm_type *tsig_algo = NULL;
 		tsig_key = read_tsig_key(
-			region, tsig_key_filename, default_family);
+			region, tsig_key_filename, default_family, &tsig_algo);
 		if (!tsig_key) {
+			error("cannot initialize TSIG: error in tsiginfo file");
 			exit(XFER_FAIL);
 		}
 
@@ -1060,8 +896,10 @@ main(int argc, char *argv[])
 
 		state.tsig = (tsig_record_type *) region_alloc(
 			region, sizeof(tsig_record_type));
-		tsig_init_record(state.tsig, region, md5, tsig_key);
+		tsig_create_record(state.tsig, region);
+		tsig_init_record(state.tsig, tsig_algo, tsig_key);
 	}
+#endif /* TSIG */
 
 	mysigaction.sa_handler = to_alarm;
 	sigfillset(&mysigaction.sa_mask);
@@ -1076,23 +914,21 @@ main(int argc, char *argv[])
 	hints.ai_protocol = IPPROTO_TCP;
 
 	if (local_hostname) {
-		int rc = getaddrinfo(local_hostname, NULL,
+		int rc = getaddrinfo(local_hostname, local_port,
 				     &hints, &local_addresses);
 		if (rc) {
 			error("local hostname '%s' not found: %s",
-			      local_hostname,
-			      gai_strerror(rc));
+				local_hostname, gai_strerror(rc));
 		}
 	}
 
-	for (; *argv; ++argv) {
+	for (/*empty*/; *argv; ++argv) {
 		/* Try each server separately until one succeeds.  */
 		int rc;
 
 		rc = getaddrinfo(*argv, port, &hints, &res0);
 		if (rc) {
-			warning("skipping bad address %s: %s\n",
-				*argv,
+			warning("skipping bad address %s: %s\n", *argv,
 				gai_strerror(rc));
 			continue;
 		}
@@ -1116,7 +952,7 @@ main(int argc, char *argv[])
 			 * address.
 			 */
 			local_address = find_by_address_family(local_addresses,
-							       res->ai_family);
+				res->ai_family);
 			if (local_addresses && !local_address) {
 				/* Continue with next remote address.  */
 				continue;
@@ -1131,14 +967,12 @@ main(int argc, char *argv[])
 			}
 
 			/* Bind socket to local address, if required.  */
-			if (local_address
-			    && bind(state.s,
-				    local_address->ai_addr,
-				    local_address->ai_addrlen) < 0)
+			if (local_address && bind(state.s,
+				local_address->ai_addr,
+				local_address->ai_addrlen) < 0)
 			{
 				warning("cannot bind to %s: %s\n",
-					local_hostname,
-					strerror(errno));
+					local_hostname, strerror(errno));
 			}
 
 			if (connect(state.s, res->ai_addr, res->ai_addrlen) < 0)
@@ -1198,6 +1032,21 @@ main(int argc, char *argv[])
 	exit(XFER_FAIL);
 }
 
+void
+get_hostname_port_frm_str(char* arg, const char** hostname,
+	const char** port)
+{
+	/* parse -a src[@port] option */
+	char* delim = strchr(arg, '@');
+
+	if (delim) {
+		*delim = '\0';
+		*port = delim+1;
+	}
+	*hostname = arg;
+}
+
+
 int
 check_matching_address_family(struct addrinfo *a0, struct addrinfo *b0)
 {
