@@ -89,23 +89,24 @@ init_notify_send(rbtree_t* tree, netio_type* netio, region_type* region,
 		sizeof(struct xfrd_soa));
 	memset(not->current_soa, 0, sizeof(struct xfrd_soa));
 	if(dbzone && dbzone->soa_rrset && dbzone->soa_rrset->rrs) {
-		xfrd_copy_soa(not->current_soa, dbzone->soa_rrset->rrs);
+		xfrd_copy_soa(not->current_soa,	dbzone->soa_rrset->rrs);
 	}
 
 	not->is_waiting = 0;
 	not->notify_send_handler.fd = -1;
 	not->notify_send_handler.timeout = 0;
 	not->notify_send_handler.user_data = not;
-	not->notify_send_handler.event_types = NETIO_EVENT_READ|NETIO_EVENT_TIMEOUT;
+	not->notify_send_handler.event_types = 
+		NETIO_EVENT_READ|NETIO_EVENT_TIMEOUT;
 	not->notify_send_handler.event_handler = xfrd_handle_notify_send;
-	netio_add_handler(netio, &not->notify_send_handler);
+		netio_add_handler(netio, &not->notify_send_handler);
 
 #ifdef TSIG
-	tsig_create_record_custom(&not->notify_tsig, region, 0, 0, 4);
+		tsig_create_record_custom(&not->notify_tsig, region, 0, 0, 4);
 #endif /* TSIG */
-	not->notify_current = 0;
+		not->notify_current = 0;
 
-	rbtree_insert(tree, (rbnode_t*)not);
+		rbtree_insert(tree, (rbnode_t*)not);
 }
 
 static int
